package com.impetus.kundera.dao;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.UUID;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;
import javax.persistence.Query;

import com.impetus.client.cassandra.common.CassandraConstants;
import com.impetus.kundera.entities.Department;
import com.impetus.kundera.entities.Employee;

public class PersistenceDAO {

	// emp and dept id as uuid

	// CREATE TABLE employee (employee_id text,employee_name text,dept_id
	// text,designation text,experience int,project text,PRIMARY KEY
	// (employee_id, employee_name));
	// CREATE TABLE department (dept_id text,dept_code text,dept_name
	// text,PRIMARY KEY (dept_id));

	// CREATE TABLE department (dept_id bigint,dept_code text,dept_name
	// text,PRIMARY KEY (dept_id) );
	// CREATE TABLE employee (employee_id bigint,dept_id bigint,designation
	// text,employee_name text,experience int,project text,PRIMARY KEY
	// (employee_id) );
	// CREATE TABLE kundera_sequences (key text,sequence_value counter,PRIMARY
	// KEY (key));

	EntityManagerFactory emf;
	EntityManager em;

	public PersistenceDAO() {
		this("cassandra-pu");
	}

	public PersistenceDAO(String persistenceUnitName) {
		try {
			Map<String, Object> propertyMap = new HashMap<String, Object>();
			propertyMap.put(CassandraConstants.CQL_VERSION,
					CassandraConstants.CQL_VERSION_3_0);
			emf = Persistence.createEntityManagerFactory(persistenceUnitName,
					propertyMap);
			em = emf.createEntityManager();
		} catch (Exception e) {
			e.printStackTrace();
			System.exit(0);
		}
	}

	public void addPerson(Object person) {
		em.persist(person);
		System.out.println("Persisted: " + person);
	}

	public void addEmployee(Object employee) {
		em.persist(employee);
		System.out.println("Persisted: " + employee);
	}

	public void addDeptAndEmployee(Object department) {
		em.persist(department);
		System.out.println("Persisted: " + department);
	}

	public void persist(Object obj) {
		em.persist(obj);
		System.out.println("Persisted: " + obj);
	}

	public void insertEmployeeOnly() {

		Department d = new Department();
		d.setDeptCode("101");
		// d.setDeptId(UUID.randomUUID().toString());
		d.setDeptName("eComm");

		Employee e1 = new Employee();
		e1.setCurrentProject("Proj1");
		e1.setDesignation("AST1");
		// e1.setEmployeeId(UUID.randomUUID().toString());
		e1.setEmployeeName("Subash1");
		e1.setExperience(1);

		Employee e2 = new Employee();
		e2.setCurrentProject("Proj2");
		e2.setDesignation("ITA");
		// e2.setEmployeeId(UUID.randomUUID().toString());
		e2.setEmployeeName("Laxmi");
		e2.setExperience(2);

		Employee e3 = new Employee();
		e3.setCurrentProject("Proj3");
		e3.setDesignation("ITA");
		// e3.setEmployeeId(UUID.randomUUID().toString());
		e3.setEmployeeName("Laxmi");
		e3.setExperience(2);

		Employee e4 = new Employee();
		e4.setCurrentProject("Proj4");
		e4.setDesignation("ITA");
		// e4.setEmployeeId(UUID.randomUUID().toString());
		e4.setEmployeeName("Laxmi");
		e4.setExperience(2);

		em.persist(e1);
		em.persist(e2);
		em.persist(e3);
		em.persist(e4);

		System.out.println("Persisted: " + e1);
	}

	public void insertEmployeeAndDept() {

		Department d = new Department();
		d.setDeptCode("101");
		// d.setDeptId(UUID.randomUUID().toString());
		d.setDeptName("eComm");

		Employee e1 = new Employee();
		e1.setCurrentProject("Proj1");
		e1.setDesignation("AST1");
		// e1.setEmployeeId(UUID.randomUUID().toString());
		e1.setEmployeeName("Subash1");
		e1.setExperience(1);

		Employee e2 = new Employee();
		e2.setCurrentProject("Proj2");
		e2.setDesignation("AST2");
		// e2.setEmployeeId(UUID.randomUUID().toString());
		e2.setEmployeeName("Subash2");
		e2.setExperience(2);

		Employee e3 = new Employee();
		e3.setCurrentProject("Proj3");
		e3.setDesignation("AST3");
		// e3.setEmployeeId(UUID.randomUUID().toString());
		e3.setEmployeeName("Subash3");
		e3.setExperience(3);

		Employee e4 = new Employee();
		e4.setCurrentProject("Proj4");
		e4.setDesignation("AST4");
		// e4.setEmployeeId(UUID.randomUUID().toString());
		e4.setEmployeeName("Subash4");
		e4.setExperience(4);

		List<Employee> el = new ArrayList<Employee>();
		el.add(e1);
		el.add(e2);
		el.add(e3);
		el.add(e4);

		d.setEmployeeList(el);

		em.setProperty(CassandraConstants.CQL_VERSION,
				CassandraConstants.CQL_VERSION_3_0);
		em.persist(d);

		System.out.println("Persisted: " + e1);
	}

	public void executeNamedQuery(String query) {
		Query jpaQuery = em.createNamedQuery(query);
		jpaQuery.setParameter(1, 245698); // set parameter
		jpaQuery.setParameter(2, 1); // set parameter
		jpaQuery.setMaxResults(10); // To Apply LIMIT

		List<?> results = jpaQuery.getResultList();
	}

	public Object getEmployee(Integer id) {
		Object person = em.find(Employee.class, id);
		return person;
	}

	public Query getQuery(String query) {
		Query q = em.createQuery(query);
		// q.setMaxResults(1000000);// the default is 100
		return q;
	}

	public List<?> getAll() {
		return getResultList("select p from Employee p");
	}

	public List<?> getResultList(String query) {
		Query q = getQuery(query);
		List<?> resultList = q.getResultList();
		return resultList;
	}

	public void delete(Object person) {
		em.remove(person);
	}

	public void merge(Object person) {
		em.merge(person);
	}

	public void close() {
		em.close();
		emf.close();
	}

	public List<?> retrieveEmployee(final String empName) {
		Query query = getNamedQueryInstance("retrieveEmployee",
				new String[] { "empName" }, new Object[] { empName });
		query.setMaxResults(100);
		return query.getResultList();
	}

	public List<?> retrieveDept(final String empName) {
		Query query = getNamedQueryInstance("retrieveDepartment",
				new String[] { "depName" }, new Object[] { empName });
		query.setMaxResults(100);
		return query.getResultList();
	}

	public Query getNamedQueryInstance(final String queryName,
			final String[] paramNames, final Object... values) {
		Query queryObject = em.createNamedQuery(queryName, Employee.class);
		if (values != null) {
			for (int i = 0; i < values.length; i++) {
				applyNamedParameterToQuery(queryObject, paramNames[i],
						values[i]);
			}
		}
		return queryObject;
	}

	private void applyNamedParameterToQuery(Query queryObject,
			String paramName, Object value) {

		queryObject.setParameter(paramName, value);
	}

	public static void main(String args[]) {
		PersistenceDAO dao = new PersistenceDAO();

		dao.insertEmployeeAndDept();

		// Department d = new Department();
		// d.setDeptCode("101");
		// // d.setDeptId(9);
		// d.setDeptName("eComm");
		//
		// Employee e1 = new Employee();
		// e1.setCurrentProject("CAS POC");
		// e1.setDesignation("AST");
		// // e1.setEmployeeId(1);
		// e1.setEmployeeName("Subash");
		// e1.setExperience(1);
		//
		// Employee e2 = new Employee();
		// e2.setCurrentProject("CAS POC");
		// e2.setDesignation("ITA");
		// // e2.setEmployeeId(2);
		// e2.setEmployeeName("Laxmi");
		// e2.setExperience(2);
		//
		// List<Employee> l = new ArrayList<Employee>();
		// l.add(e1);
		// l.add(e2);
		//
		// d.setEmployeeList(l);
		//
		// Map<String, Object> propertyMap = new HashMap<String, Object>();
		// propertyMap.put(CassandraConstants.CQL_VERSION,
		// CassandraConstants.CQL_VERSION_3_0);
		// EntityManagerFactory emf = Persistence.createEntityManagerFactory(
		// "cassandra-pu", propertyMap);
		// EntityManager em = emf.createEntityManager();
		//
		// // em.persist(d);
		// List<?> l1 = dao.getAll();
		// System.out.println(l1);
		// List<?> l2 =
		// dao.getQuery("select p from Department p").getResultList();
		// System.out.println(l2);
		//
		// e1.setEmployeeName("SBangaru");
		// try {
		// em.merge(d);
		// } catch (Exception exp) {
		// exp.printStackTrace();
		// }
		//
		// try {
		// Query jpaQuery = em.createNamedQuery("retrieveDepartment",
		// Department.class);
		// jpaQuery.setParameter(1, "eComm"); // set parameter
		// jpaQuery.setMaxResults(10); // To Apply LIMIT
		//
		// List<?> results = jpaQuery.getResultList();
		// System.out.println();
		// } catch (Exception exp) {
		// exp.printStackTrace();
		// }
		//
		// try {
		// Query jpaQuery = em.createNamedQuery("retrieveEmployee",
		// Employee.class);
		// jpaQuery.setParameter(1, "Subash"); // set parameter
		// jpaQuery.setMaxResults(10); // To Apply LIMIT
		//
		// List<?> results = jpaQuery.getResultList();
		// } catch (Exception exp) {
		// exp.printStackTrace();
		// }
		// try {
		// Query jpaQuery = em.createNamedQuery("findDeptById",
		// Department.class);
		// jpaQuery.setParameter(1, 9); // set parameter
		// jpaQuery.setMaxResults(10); // To Apply LIMIT
		//
		// List<?> results = jpaQuery.getResultList();
		// } catch (Exception exp) {
		// exp.printStackTrace();
		// }
		//
		// try {
		// Query jpaQuery = em.createNamedQuery("findEmployeeById",
		// Employee.class);
		// jpaQuery.setParameter(1, 2); // set parameter
		// jpaQuery.setMaxResults(10); // To Apply LIMIT
		//
		// List<?> results = jpaQuery.getResultList();
		// } catch (Exception exp) {
		// exp.printStackTrace();
		// }
		dao.close();
	}
}
